#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>

#include "sbutils.h"
#include "sbawsdatastorage.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

TElAWSS3DataStorage m_DataStorage (NULL);
TElHTTPSClient m_Client (NULL);
TElX509Certificate m_Cert (NULL);

void SB_CALLBACK SSecurityHandlerCreatedEvent(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TElCustomDataStorageObjectHandle Obj, void * Data, TElCustomDataStorageSecurityHandlerHandle hHandler)
{
    TElMemoryCertStorage encryptionCerts (NULL);
    encryptionCerts.Add(m_Cert, false);

    TElDefaultDataStorageSecurityHandler handler (hHandler, false);
    handler.set_EncryptionCertificates(encryptionCerts);
    encryptionCerts.updateHandle(NULL);
    handler.set_KeyEncryptionAlgorithm(SB_ALGORITHM_CNT_AES128);
    handler.set_DataEncryptionAlgorithm(SB_ALGORITHM_CNT_AES128);
    handler.set_AuthInfoType(aitNone);
}

void SB_CALLBACK DataStorageProgressEvent(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TSBDataStorageOperationRaw Operation, int64_t Total, int64_t Current, int8_t * Cancel)
{
	int percent = Current * 100 / Total;
	std::cout << percent << "%" << "\r";
}

void SB_CALLBACK CertificateValidateEvent(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TElX509CertificateHandle X509Certificate, int8_t * Validate)
{
    *Validate = true;
	// NEVER do this in production code. Instead perform full certificate validation.
	// You can use TElX509CertificateValidator class for complete certificiate chain validation. 
	// See other samples for example of how TElX509CertificateValidator class should be used.
}

void ShowBucketList()
{
    try
    {
		std::cout << "Bucket list:" << std::endl;

		TStringList conts;
		m_DataStorage.ListBuckets(conts);
		for (int i = 0; i < conts.get_Count(); i++)
        {
			std::string container;
			conts.get_Strings(i, container);
			std::cout << container << std::endl;
        }
    }
    catch (SBException e)
    {
       std::cout << e.what() << std::endl;
    }
}

void ShowObjectList(std::string container)
{
	try
	{
		std::cout << "Object list for " << container << ":" << std::endl;

		TElDataStorageObjectList objs;
		m_DataStorage.List(container, objs);
		for (int i = 0; i < objs.get_Count(); i++)
		{
			TElCustomDataStorageObject* obj = objs.get_Objects(i);

			std::string name;
			obj->get_Name(name);

			std::cout << name << std::endl;
		}
	}
    catch (SBException e)
    {
       std::cout << e.what() << std::endl;
    }
}

std::string GetFilename(std::string const& path)
{
	std::string result;
	size_t pos = path.find_last_of("\\");
	if(pos != std::string::npos)
		result.assign(path.begin() + pos + 1, path.end());
	else
		result = path;

	return result;
}

void WriteObject(std::string containerName, std::string path)
{
	TFileStream inputStream (path, filemodeOpenRead);
    try
    {
        TElDefaultDataStorageSecurityHandler handler (NULL);
        TElMemoryCertStorage encryptionCerts (NULL);
        encryptionCerts.Add(m_Cert, false);
        handler.set_EncryptionCertificates(encryptionCerts);
        handler.set_KeyEncryptionAlgorithm(SB_ALGORITHM_CNT_AES128);
        handler.set_DataEncryptionAlgorithm(SB_ALGORITHM_CNT_AES128);
		handler.set_AuthInfoType(aitNone);

		std::cout << "Uploading object " << path  << std::endl;

		m_DataStorage.WriteObject(containerName, GetFilename(path), inputStream, handler);
    }
    catch (SBException e)
    {
		std::cout << "Object write error: " << e.what() << std::endl;
    }
}

void ReadObject(std::string containerName, std::string name, std::string path)
{
	TFileStream outputStream (path + name, filemodeCreate);
    try
    {
		std::cout << "Downloading object " << path + name << std::endl;
		m_DataStorage.ReadObject(containerName, name, outputStream);
    }
    catch (SBException e)
    {
		std::cout << "Error: " << e.what() << std::endl;
    }
}

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

bool LoadCertificate(std::string certFilename, std::string certPassword)
{
	TFileStream f (certFilename, filemodeOpenRead);
	int r = m_Cert.LoadFromStreamPFX(f, certPassword, 0);
    if (r != 0)
    {
        std::cout << "Failed to load certificate, error " << r << std::endl;
    }

	return r == 0;
}

bool LoginStorage(std::string id, std::string key)
{
    m_DataStorage.set_KeyID(id);
    m_DataStorage.set_AccessKey(key);
	return true;
}

int main(int argc, char **argv)
{
	std::string helpMessage = "aws.exe -a [id key][flags] [operation] -c [filename] [password]" \
		"-a [id key] : account id and key" \
		"-w [container file]: upload file to storage" \
		"-r [container name path]: download file by 'name' in 'path' location" \
		"flags:" \
		"-containerlist : show container list;" \
		"-objectlist [container] : show object in container;" \
		"-c [filename] [password] : certificate";
	int i = 0;
	bool write = false;
	bool read = false; 
	bool showBuckets = false; 
	bool showObject = false; 
	bool loadCert = false;
	std::string certFilename, certPassword;
	std::string id, key, container, filename, name, path;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
	}

	std::cout << "EldoS Win Azure console demo." << std::endl;

	m_DataStorage.set_OnSecurityHandlerCreated(SSecurityHandlerCreatedEvent, NULL);
    m_DataStorage.set_OnProgress(DataStorageProgressEvent, NULL);
	m_Client.set_OnCertificateValidate(CertificateValidateEvent, NULL);
    m_DataStorage.set_HTTPClient(m_Client);

	try
	{
		i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-a")
			{
				if (++i > argc)
					throw new SBException();

				id = UnQuote(argv[i]);

				if (++i > argc)
					throw new SBException();

				key = UnQuote(argv[i]);
			}
			else if (param == "-containerlist")
			{
				showBuckets = true;
			}
			else if (param == "-objectlist")
			{
				if (++i > argc)
					throw new SBException();

				container = UnQuote(argv[i]);
				showObject = true;
			}
			else if (param == "-w")
			{
				if (++i > argc)
					throw new SBException();

				container = UnQuote(argv[i]);	

				if (++i > argc)
					throw new SBException();

				filename = UnQuote(argv[i]);

				write = true;
				read = false;
			}
			else if (param == "-r")
			{
				if (++i > argc)
					throw new SBException();

				container = UnQuote(argv[i]);	

				if (++i > argc)
					throw new SBException();

				name = UnQuote(argv[i]);

				if (++i > argc)
					throw new SBException();

				path = UnQuote(argv[i]);
			
				read = true;
				write = false;
			}
			else if (param == "-c")
			{
				if (++i > argc)
					throw new SBException();

				certFilename = UnQuote(argv[i]);

				if (++i < argc)
					certPassword = UnQuote(argv[i]);

				loadCert = true;
			}
			else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
			}

			i++;
		}	
	}
    catch (SBException E)
    {
		std::cout << "Parameters parsing error:" << E.what() << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
    }

	if (LoginStorage(id, key))
	{
		if (showBuckets)
			ShowBucketList();
		if (showObject)
			ShowObjectList(container);
		if (loadCert)
			LoadCertificate(certFilename, certPassword);

		if (write)
			WriteObject(container, filename);
		else if (read)
			ReadObject(container, name, path);
	}

	//std::string result;
	//std::getline(std::cin, result);

	return 0;
}