#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>

#include "sbmisc.h"
#include "sbsimpleftps.h"
#include "sbx509.h"
#include "sbutils.h"

using namespace SecureBlackbox;

#define OP_LIST		1
#define OP_DOWNLOAD	2
#define OP_UPLOAD	3

void SB_CALLBACK Client_OnCertificateValidate(void * /* objData */, TObjectHandle /* hSender */, TElX509CertificateHandle hCertificate, int8_t * Validate)
{
	try
	{
		TElX509Certificate Cert(hCertificate, false);
		TMessageDigest160 M160;
		Cert.GetHashSHA1(M160);
		std::string dgst;
		DigestToStr(M160, true, dgst);
		std::cout << "Server certificate received, fingerprint " << dgst << std::endl;
		*Validate = true; // NEVER do this. You MUST check the key validity somehow
	}
	catch (SBException E)
	{
		std::cout << "Unexpected error in OnCertificateValidate event handler" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;
		*Validate = false;
	}
}

void SB_CALLBACK Client_OnTextDataLine(void * objData, TObjectHandle /* hSender */, const uint8_t pcTextLine[], int32_t szTextLine)
{
	std::string response((char *)pcTextLine, szTextLine);

	try
	{
		TElFTPFileInfo FileInfo;
		TElSimpleFTPSClient *Client = (TElSimpleFTPSClient *)objData;
		if (Client->get_ExtMLSTSupported())
			TElSimpleFTPSClient::ParseMLSDEntry(response, FileInfo);
		else
			TElSimpleFTPSClient::ParseFileListEntry(response, FileInfo);

		std::string filename;
		FileInfo.get_Name(filename);
		std::cout << filename << std::endl;
	}
	catch (SBException E)
	{
		// Couldn't parse the entry, so just write what we've received
		std::cout << response << std::endl;
	}
}

void Download(TElSimpleFTPSClient &Client, const std::string &remoteName, const std::string &localName)
{
	std::ofstream fs;
	fs.open(localName.c_str(), std::ios::binary | std::ios::out | std::ios::trunc);
	if (fs.is_open())
	{
		TElCallbackStream cs(fs);
		Client.Receive(remoteName, cs, 0, 0);
		std::cout << "File " << remoteName << " received to " << localName << std::endl;
		fs.close();
	}
	else
		std::cout << "Unable to open file: " << localName << std::endl;
}

void Upload(TElSimpleFTPSClient &Client, const std::string &localName, const std::string &remoteName)
{
	std::ifstream fs;
	fs.open(localName.c_str(), std::ios::binary | std::ios::in);
	if (fs.is_open())
	{
		TElCallbackStream cs(fs);
		Client.Send(cs, remoteName);
		std::cout << "File " << localName << " send to " << remoteName << std::endl;
		fs.close();
	}
	else
		std::cout << "Unable to open file: " << localName << std::endl;
}

void List(TElSimpleFTPSClient &Client)
{
	std::cout << "Requesting list" << std::endl;
	if (Client.get_ExtMLSTSupported())
		Client.MLSD();
	else
		Client.GetFileList();
}

int main(int argc, char **argv)
{
	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	if (argc < 3)
	{
		std::cout << "Usage:" << std::endl;
		std::cout << "ConsoleFTPS {-download|-upload|-list} {-passive} {-ssl} -addr[ess] <address> [-port <port>] -user[name] <user> -pass[word] <password> -remote <remote file name> -local <local file name>" << std::endl;
		return 0;
	}

	int paramOperation = 0;
	int paramPort = 21;
	bool paramPassiveMode = false;
	bool paramUseSSL = false;
	std::string paramAddress;
	std::string paramUsername;
	std::string paramPassword;
	std::string paramLocalName;
	std::string paramRemoteName;

	int i = 1;
	while (i < argc)
	{
		char *arg = argv[i];
		if ((!strcmp(arg, "-addr") ||
			!strcmp(arg, "-address")) && (i < argc - 1))
			paramAddress.assign(argv[++i]);
		else if (!strcmp(arg, "-port") && (i < argc - 1))
		{
			paramPort = atoi(argv[++i]);
			if (0 == paramPort)
				paramPort = 22;
		}
		else if ((!strcmp(arg, "-user") ||
			!strcmp(arg, "-username")) && (i < argc - 1))
			paramUsername.assign(argv[++i]);
		else if ((!strcmp(arg, "-pass") ||
			!strcmp(arg, "-password")) && (i < argc - 1))
			paramPassword.assign(argv[++i]);
		else if (!strcmp(arg, "-down") ||
			!strcmp(arg, "-download"))
			paramOperation = OP_DOWNLOAD;
		else if (!strcmp(arg, "-up") ||
			!strcmp(arg, "-upload"))
			paramOperation = OP_UPLOAD;
		else if (!strcmp(arg, "-passive"))
			paramPassiveMode = true;
		else if (!strcmp(arg, "-ssl"))
			paramUseSSL = true;
		else if (!strcmp(arg, "-ls") ||
			!strcmp(arg, "-list"))
			paramOperation = OP_LIST;
		else if (!strcmp(arg, "-local") && (i < argc - 1))
			paramLocalName.assign(argv[++i]);
		else if (!strcmp(arg, "-remote") && (i < argc - 1))
			paramRemoteName.assign(argv[++i]);

		i++;
	}

	TElSimpleFTPSClient Client(NULL);
	Client.set_Address(paramAddress);
	Client.set_Port(paramPort);

	Client.set_Username(paramUsername);
	Client.set_Password(paramPassword);

	Client.set_OnCertificateValidate(&Client_OnCertificateValidate, NULL);
	Client.set_OnTextDataLine(&Client_OnTextDataLine, &Client);

	Client.set_UseSSL(paramUseSSL);
	Client.set_PassiveMode(paramPassiveMode);

	try
	{
		std::cout << "Connecting " << paramAddress << ":" << paramPort << std::endl;
		Client.Open();
		std::string lastReply = "";
		Client.Login(lastReply);
	}
	catch (SBException E)
	{
		std::cout << "Connection failed due to exception:" << std::endl;
		std::cout << E.what() << std::endl;
		std::cout << "Stack trace: " << E.getErrorStackTrace().c_str() << std::endl;

		std::cout << "If you have ensured that all connection parameters are correct and you still can't connect," << std::endl;
		std::cout << "please contact EldoS support as described on http://www.eldos.com/sbb/support-tech.php" << std::endl;
		std::cout << "Remember to provide details about the error that happened." << std::endl;

		try
		{
			Client.Close(true);
		}
		catch (...)
		{
		}

		return 0;
	}

	try
	{
		switch (paramOperation)
		{
		case OP_DOWNLOAD:
			if (paramRemoteName == "")
			{
				std::cout << "Remote filename is not specified" << std::endl;
				break;
			}

			if (paramLocalName == "")
				paramLocalName = paramRemoteName;

			Download(Client, paramRemoteName, paramLocalName);
			break;

		case OP_UPLOAD:
			if (paramLocalName == "")
			{
				std::cout << "Local filename is not specified" << std::endl;
				break;
			}

			if (paramRemoteName == "")
				paramRemoteName = paramLocalName;

			Upload(Client, paramLocalName, paramRemoteName);
			break;

		default:
			List(Client);
			break;
		}
	}
	catch (SBException E)
	{
		std::cout << E.what() << std::endl;
	}

	try
	{
		Client.Close(false);
	}
	catch (SBException E)
	{
		std::cout << E.what() << std::endl;
	}

	return 0;
}
