#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "sbutils.h"
#include "sbcertvalidator.h"
#include "sbhttpsclient.h"
#include "sbhttpcrl.h"
#include "sbhttpocspclient.h"
#include "sbhttpcertretriever.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

TElHTTPSClient HTTPSClient (NULL);
TElX509CertificateValidator CertificateValidator (NULL);

std::string UnQuote(std::string s)
{
    if (s.length() < 1)
        return s;

	s.erase(remove( s.begin(), s.end(), '\"' ), s.end());

    return s;
}

void SB_CALLBACK OnBeforeOCSPClientUse(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TElX509CertificateHandle Certificate, TElX509CertificateHandle CACertificate, const char * pcOCSPLocation, int32_t szOCSPLocation, TElOCSPClientHandle * OCSPClient)
{
	TElHTTPOCSPClient client(*OCSPClient, false);
	std::string proxyHost, proxyUsername, proxyPassword;
	HTTPSClient.get_HTTPProxyHost(proxyHost);
	HTTPSClient.get_HTTPProxyUsername(proxyUsername);
	HTTPSClient.get_HTTPProxyPassword(proxyPassword);
	client.get_HTTPClient()->set_UseHTTPProxy(HTTPSClient.get_UseHTTPProxy());
	client.get_HTTPClient()->set_HTTPProxyHost(proxyHost);
	client.get_HTTPClient()->set_HTTPProxyPort(HTTPSClient.get_HTTPProxyPort());
	client.get_HTTPClient()->set_HTTPProxyUsername(proxyUsername);
	client.get_HTTPClient()->set_HTTPProxyPassword(proxyPassword);
}

void SB_CALLBACK OnBeforeCRLRetrieverUse(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle Certificate, TElX509CertificateHandle CACertificate, TSBGeneralNameRaw NameType, const char * pcLocation, int32_t szLocation, TElCustomCRLRetrieverHandle * Retriever)
{
	TElHTTPCRLRetriever retriever(*Retriever, false);
	std::string proxyHost, proxyUsername, proxyPassword;
	HTTPSClient.get_HTTPProxyHost(proxyHost);
	HTTPSClient.get_HTTPProxyUsername(proxyUsername);
	HTTPSClient.get_HTTPProxyPassword(proxyPassword);
	retriever.get_HTTPClient()->set_UseHTTPProxy(HTTPSClient.get_UseHTTPProxy());
	retriever.get_HTTPClient()->set_HTTPProxyHost(proxyHost);
	retriever.get_HTTPClient()->set_HTTPProxyPort(HTTPSClient.get_HTTPProxyPort());
	retriever.get_HTTPClient()->set_HTTPProxyUsername(proxyUsername);
	retriever.get_HTTPClient()->set_HTTPProxyPassword(proxyPassword);
}

void SB_CALLBACK OnDocumentBegin(void * /*_ObjectData*/, TObjectHandle /*Sender*/)
{
	std::cout << "-- Document started --" << std::endl;
}

void SB_CALLBACK OnDocumentEnd(void * /*_ObjectData*/, TObjectHandle /*Sender*/)
{
	std::cout << "-- Document finished --" << std::endl;
}

void SB_CALLBACK OnCloseConnection(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TSBCloseReasonRaw CloseReason)
{
	std::cout << "-- Connection closed --" << std::endl;
}

void SB_CALLBACK OnData(void * /*_ObjectData*/, TObjectHandle /*Sender*/, void * Buffer, int32_t Size)
{
	std::string data((char*)Buffer, Size);
	std::cout << data << std::endl;
}

void SB_CALLBACK OnPreparedHeaders(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TStringListHandle Headers)
{
	TStringList list (Headers, false);
	std::string s;
	list.get_Text(s);
	std::cout << "Sending headers: " << s << std::endl;
}

void SB_CALLBACK OnReceivingHeaders(void * /*_ObjectData*/, TObjectHandle /*Sender*/, TStringListHandle Headers)
{
	TStringList list(Headers, false);
	std::string s;
	list.get_Text(s);
	std::cout << "Received headers: " << s << std::endl;
}

void SB_CALLBACK OnRedirection(void * /*_ObjectData*/, TObjectHandle /*Sender*/, const char * pcOldURL, int32_t szOldURL, char * pcNewURL, int32_t * szNewURL, int8_t * AllowRedirection)
{
	std::string data(pcNewURL, *szNewURL);
	std::cout << "Redirected to " << data << std::endl;
	*AllowRedirection = true;
}

void SB_CALLBACK OnCertificateValidate(void * _ObjectData, TObjectHandle Sender, TElX509CertificateHandle X509Certificate, int8_t * Validate)
{
	TElX509Certificate Certificate (X509Certificate, false);
	TSBCertificateValidity Validity = cvInvalid;
	TSBCertificateValidityReason Reason;
	if ((Certificate.get_Chain() == NULL) || (Certificate.get_Chain()->get_Certificates(0)->getHandle() == X509Certificate))
	{
		// For proper CRL and OCSP validation please read instructions in 
		// description of ElX509CertificateValidator class in the help file
		std::string remoteHost, remoteIP;
		HTTPSClient.get_RemoteHost(remoteHost);
		HTTPSClient.get_RemoteIP(remoteIP);
		std::time_t t = std::time(0);
		CertificateValidator.ValidateForSSL(&Certificate, remoteHost, remoteIP, hrServer, NULL, false, t, Validity, Reason);
		*Validate = (Validity == cvOk) || (Validity == cvSelfSigned);
	}
	else		
		*Validate = true; 
}

int main(int argc, char **argv)
{
	int i;
	std::string helpMessage = "consoleHTTPSGet.exe [params]\n" \
		"[params]\n" \
		"-protocol : HTTP or HTTPS (deafult : HTTP);\n" \
		"-host : server url (default : www.eldos.com);\n" \
		"-port : server port (default : 80);\n" \
		"-path : server path (default : '/');\n" \
		"-out : path to save the results (default: not set. Only output in console)\n";
	std::string protocol = "HTTP";
	std::string host = "www.eldos.com";
	std::string port = "80";
	std::string path = "/";
	std::string outputFilename;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	try
	{
		i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-protocol")
			{
				if (++i <= argc)
				{
					std::string value = UnQuote(argv[i]);
					std::transform(value.begin(), value.end(), value.begin(), ::toupper);

					if (value == "HTTP")
					{
						protocol = "HTTP";
					}
					else if (value == "HTTPS")
					{
						protocol = "HTTPS";
					}
					else
					{
						std::cout << "Use the correct protocol." << std::endl;
						return 0;
					}

				}
			}
			else if (param == "-host")
			{
				if (++i <= argc)
					host = UnQuote(argv[i]);
			}
			else if (param == "-port")
			{
				if (++i <= argc)
					host = UnQuote(argv[i]);
			}
			else if (param == "-path")
			{
				if (++i <= argc)
					path = UnQuote(argv[i]);
			}
			else if (param == "-out")
			{
				if (++i <= argc)
					outputFilename = UnQuote(argv[i]);
			}

			i++;
		}
	}
	catch (...)
	{
		std::cout << "Failed to parse parameters" << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
		RegisterHTTPCRLRetrieverFactory();
		RegisterHTTPOCSPClientFactory();
		RegisterHTTPCertificateRetrieverFactory();

		CertificateValidator.InitializeWinStorages();
		CertificateValidator.set_OnBeforeOCSPClientUse(&OnBeforeOCSPClientUse, NULL);
		CertificateValidator.set_OnBeforeCRLRetrieverUse(&OnBeforeCRLRetrieverUse, NULL);

		HTTPSClient.set_OnDocumentBegin(&OnDocumentBegin, NULL);
		HTTPSClient.set_OnDocumentEnd(&OnDocumentEnd, NULL);
		HTTPSClient.set_OnCloseConnection(&OnCloseConnection, NULL);
		if (outputFilename == "")
			HTTPSClient.set_OnData(&OnData, NULL);

		HTTPSClient.set_OnPreparedHeaders(&OnPreparedHeaders, NULL);
		HTTPSClient.set_OnReceivingHeaders(&OnReceivingHeaders, NULL);
		HTTPSClient.set_OnRedirection(&OnRedirection, NULL);
		HTTPSClient.set_OnCertificateValidate(&OnCertificateValidate, NULL);

		std::string URL = protocol + "://" + host + ":" + port + path;

		TFileStream* Stream = NULL;
		if (outputFilename != "")
		{
			try
			{
				Stream = new TFileStream(outputFilename, filemodeCreate);
			}
			catch (SBException E)
			{
				std::cout << "Failed to create output stream." << std::endl;
			}

			HTTPSClient.set_OutputStream(Stream);
		}

		try
		{
			HTTPSClient.Get(URL);
		}
		catch (SBException E)
		{
			std::cout << "Exception happened during HTTP download: " << E.what() << std::endl;
		}

		HTTPSClient.Close(true);
		if (Stream != NULL)
		{
			HTTPSClient.set_OutputStream(NULL);
			delete Stream;
		}
	}
	catch (SBException ex)
	{
		std::cout << "Error" << ex.what() << std::endl;
		return 0;
	}
}