//
//  ServerThread.h
//  OSXHTTPServer
//
//  Created by EldoS on 13.03.15.
//  Copyright (c) 2015 EldoS. All rights reserved.
//

#ifndef __OSXHTTPServer__ServerThread__
#define __OSXHTTPServer__ServerThread__

#include <stdio.h>
#include <thread>

#include "sbsocket.h"
#include "sbhttpscommon.h"
#include "sbhttpsserver.h"
#include "sbtypes.h"
#include "sbutils.h"
#include "sbsslconstants.h"
#include "sbsslcommon.h"
#include "sbsslserver.h"
#include "sbsessionpool.h"
#include "sbconstants.h"
#include "sbcustomcertstorage.h"
#include "sbsharedresource.h"
#include "sbstrutils.h"
#include "sbwebsocketcommon.h"
#include "sbwebsocketserver.h"
#include "sbsslclient.h"

using namespace SecureBlackbox;

typedef enum
{
    hcaGZip = 0,
    hcaDeflate = 1
} TSBHTTPCompressionAlg;

void CompressStream(TStream *Input, TStream *Output, TSBHTTPCompressionAlg CompAlg, int CompLevel);

typedef void (*TLogEvent)(std::string Line);
typedef void (*TLogRequest)(TElHTTPServerRequestParams *Request);

void SB_CALLBACK DoSend(void * _ObjectData, TObjectHandle Sender, void * Buffer, int32_t Size);
void SB_CALLBACK DoReceive(void * _ObjectData, TObjectHandle Sender, void * Buffer, int32_t MaxSize, int32_t * Written);
void SB_CALLBACK DoCloseConnection(void * _ObjectData, TObjectHandle Sender, TSBCloseReasonRaw CloseReason);
void SB_CALLBACK DoRequestHeadersReceived(void * _ObjectData, TObjectHandle Sender, TElHTTPServerRequestParamsHandle RequestInfo, int8_t * Accepted);
void SB_CALLBACK DoData(void * _ObjectData, TObjectHandle Sender, void * Buffer, int32_t Size);
void SB_CALLBACK DoRequestBodyReceived(void * _ObjectData, TObjectHandle Sender, TElHTTPServerRequestParamsHandle RequestInfo);
void SB_CALLBACK DoOpenConnection(void * _ObjectData, TObjectHandle Sender);
void SB_CALLBACK DoError(void * _ObjectData, TObjectHandle Sender, int32_t ErrorCode, int8_t Fatal, int8_t Remote);
void SB_CALLBACK DoBeforeWebSocketServerUsed(void * _ObjectData, TObjectHandle Sender, TElWebSocketServerHandle WSServer);
void SB_CALLBACK DoWebSocketConnectionEstablished(void * _ObjectData, TObjectHandle Sender, TElWebSocketServerHandle WSServer);
void SB_CALLBACK DoTextData(void * _ObjectData, TObjectHandle Sender, const char * pcText, int32_t szText, int8_t Last);

class THTTPSServerListeningThread;
class THTTPSServerThread;

class THTTPSServerListeningThread
{
public:
    std::thread slThread;
    bool slTerminated, slFinished;
    TElSharedResource *FSharedResource;
    TList *FSessions;
    TElSocket *FServerSocket;
    std::string FBasePath;
    TElCustomCertStorage *FCertStorage;
    bool FSSLMode, FUseChunkedTransfer, FUseCompression;
    TLogEvent FOnLog;
    TLogRequest FOnRequest;
    THTTPSServerListeningThread(int ListenPort);
    ~THTTPSServerListeningThread();
    void Log(std::string Msg);
    void Execute();
    void Terminate();
    void Start();
};

class THTTPSServerThread
{
public:
    std::thread sThread;
    bool sTerminated, sFinished;
    TElWebSocketServer *FWSServer;
    THTTPSServerListeningThread *FParent;
    TElSocket *FSocket;
    TElHTTPSServer *FHTTPSServer;
    TElHTTPServerRequestParams *FCurrentRequest;
    TElHTTPServerResponseParams *FCurrentResponse;
    TFileStream *FFile;
    std::string FBasePath;
    bool FUseChunkedTransfer, FUseCompression;
    bool FUpload;
    TMemoryStream *FUploadStream;
    TLogEvent FOnLog;
    TLogRequest FOnRequest;
    THTTPSServerThread(THTTPSServerListeningThread *Parent, TElSocket *Socket);
    ~THTTPSServerThread();
    void Log(std::string Msg);
    void ProcessRequest();
    void Execute();
    void Terminate();
    void Start();
    inline void set_BasePath(std::string value);
    inline void set_SSLMode(bool value);
    inline bool get_SSLMode();
    inline void set_CertStorage(TElCustomCertStorage *value);
    inline TElCustomCertStorage *get_CertStorage();
};

#endif /* defined(__OSXHTTPServer__ServerThread__) */
