#include <cstring>
#include <iostream>
#include <fstream>
#include <stdio.h>
#include <stdlib.h>
#include <algorithm>
#include <ctime>

#include "sbutils.h"
#include "sbhttpcrl.h"
#include "sbhttpocspclient.h"
#include "sbhttpcertretriever.h"
#include "sbsslconstants.h"
#include "sbsslcommon.h"
#include "sbsimplessl.h"
#include "sbx509.h"
#include "sbcertvalidator.h"
#include "sbcustomcertstorage.h"
#include "sbmisc.h"

using namespace SecureBlackbox;

std::string UnQuote(std::string s)
{
	if (s.length() < 1)
		return s;

	s.erase(remove(s.begin(), s.end(), '\"'), s.end());
	return s;
}

void SB_CALLBACK Client_OnCertificateValidate(void * /*_ObjectData*/, TObjectHandle /* hSender */, TElX509CertificateHandle hX509Certificate, int8_t * Validate)
{
	*Validate = true;
	// NEVER do this in production code. Instead perform full certificate validation.
	// You can use TElX509CertificateValidator class for complete certificiate chain validation. 
	// See other samples for example of how TElX509CertificateValidator class should be used.
}

void DoRequest(TElSimpleSSLClient &Client, const std::string host)
{
	try
	{
		Client.Open();
		if (!Client.get_Active())
		{
			std::cout << "Failed to establish connection" << std::endl;
			return;
		}
	}
	catch (SBException E)
	{
		std::cout << "Failed to establish connection: " << E.what() << std::endl;
		return;
	}

	std::cout << "SSL connection established" << std::endl;
	std::string v;
	switch (Client.get_CurrentVersion())
	{
	case sbSSL2:
		v = "SSL2";
		break;
	case sbSSL3:
		v = "SSL3";
		break;
	case sbTLS1:
		v = "TLS1";
		break;
	case sbTLS11:
		v = "TLS1.1";
		break;
	case sbTLS12:
		v = "TLS1.2";
		break;

	default:
		v = "Unknown";
		break;
	}

	std::cout << "Version is " << v << std::endl;
	std::vector<uint8_t> Buffer;
	std::cout << "Sending HTTP request ..." << std::endl;
	std::string request = "GET / HTTP/1.1\nHost:" + host + "\nUser-Agent: EldoS SSLBlackbox (.NET edition)\nConnection: close\n\n";
	BytesOfString(request, Buffer);
	Client.SendData(&Buffer.front(), Buffer.size());

	int ToRead = 0;
	while (Client.get_Active())
	{
		try
		{
			ToRead = 0x1000;
			std::vector<uint8_t> Buffer(ToRead);
			std::string res;
			Client.ReceiveData(&Buffer[0], ToRead, true);
			Buffer.resize(ToRead);
			StringOfBytes(Buffer, res);
			std::cout << res << std::endl;
		}
		catch (SBException E)
		{
			// here we can catch disconnection or timeout event or something like this
			break;
		}
	}

	if (Client.get_Active())
		std::cout << "Disconnecting" << std::endl;

	Client.Close(false);
}

int main(int argc, char **argv)
{
	std::string helpMessage = "consoleSSLClient.exe [params]\n" \
		"params:\n" \
		"-host <hostname> : host name (default : www.microsoft.com);\n" \
		"-port <port> : port number (default : 443);\n" \
		"-ssl <yes|no>: enable or disable SSL (default : yes);\n";
	
	std::string host = "www.microsoft.com";
	int port = 443;
	bool enableSSL = true;

	try
	{
		#error Please pick the evaluation license key from <SecureBlackbox>\LicenseKey.txt file and place it here. If the evaluation key expires, you can request an extension using the form on https://www.eldos.com/sbb/keyreq/
SetLicenseKey("<put the key here>");
	}
	catch (SBException E)
	{
		std::cout << "Failed to set license key:" << std::endl;
		std::cout << E.what() << std::endl;
		return 0;
	}

	try
	{
		int i = 1;
		while (i < argc)
		{
			std::string param;
			param.assign(argv[i]);

			if (param == "-host")
			{
				if (++i < argc)
					host = UnQuote(argv[i]);
			}
			else if (param == "-port")
			{
				if (++i < argc)
				{
					std::string value = UnQuote(argv[i]);
					port = atoi(value.c_str());
				}
			}
			else if (param == "-ssl")
			{
				if (++i < argc)
				{
					std::string value = UnQuote(argv[i]);
					if (value == "yes")
					{
						enableSSL = true;
					}
					else if (value == "no")
					{
						enableSSL = false;
					}
					else
					{
						std::cout << "SSL mode incorrect." << std::endl;
						return 0;
					}
				}
			}
			else if (param == "-h")
			{
				std::cout << helpMessage << std::endl;
				return 0;
			}

			i++;
		}
	}
	catch (...)
	{
		std::cout << "Failed to parse parameters" << std::endl;
		std::cout << helpMessage << std::endl;
		return 0;
	}

	try
	{
		RegisterHTTPCRLRetrieverFactory();
		RegisterHTTPOCSPClientFactory();
		RegisterHTTPCertificateRetrieverFactory();

		TElMemoryCertStorage clientCerts(NULL);

		TElX509CertificateValidator CertificateValidator(NULL);
		CertificateValidator.InitializeWinStorages();

		TElSimpleSSLClient SSLClient(NULL);
		SSLClient.set_OnCertificateValidate(&Client_OnCertificateValidate, NULL);

		std::cout << "Connecting to " << host << "..." << std::endl;
		/*
		int certIndex = -1;
		TSBObjectList* objs = clientCerts.get_CertificateList();
		for (int i = 0; i < objs->get_Count(); i++) {
			int index = clientCerts.get_Chains(i);
			if (clientCerts.get_Certificates(index)->get_PrivateKeyExists())
			{
				certIndex = index;
				break;
			}
		}
		*/

		SSLClient.set_Address(host);
		SSLClient.set_Port(port);
		SSLClient.set_Enabled(enableSSL);
		SSLClient.set_Versions((TSBVersions)(f_sbTLS1 | f_sbTLS11 | f_sbTLS12));

		DoRequest(SSLClient, host);
	}
	catch (SBException E)
	{
		std::cout << "Error: " << E.what() << std::endl;
		return 0;
	}
}